/*
 * Example for Secure 4 click
 *
 * Date         Jun 2017
 * Author       Djordje Rosic
 *
 * Test configuration PIC:
 *  MCU           :        P18F87K22
 *  Dev. Board    :        EasyPIC PRO v7
 *  SW            :        mikroC PRO for PIC v7.1.0
 *
 * Description :
 *
 *   This example demonstrates the use of Secure 4 click.
 *
 *   IMPORTANT NOTICE: Most features of this device require config or data
 *   zones of ECC608A device to be locked. After a zone is locked, it cannot
 *   be unlocked again. This security measure means it is not possible to
 *   reprogram the device for a different purpose, once it is configured.
 *
 *   This example will only use the functions that won't require locking
 *   any of the data zones, so that any change they make to the device is not
 *   permanent. Demonstration of some of the more advanced functions is
 *   included in the comments, but take great care when using those, as
 *   they will make irreversible changes to your device. Consult the device
 *   datasheet for more information.
 *
 *       Note: Data output is via UART, turn on the required switches on
 *   the EasyPIC board.
 *       All the basic functions for this device are prefixed with atcab_
 *   and are contained in the atca_basic.c file.
 *
 ******************************************************************************/

#include "secure_4_driver.c"
#define LOG(x) UART1_Write_Text(x)

uint8_t bufferIn  [ 128 ];
uint8_t bufferOut [ 128 ];

static void additionalTests ();
static void signatureTest ();
void outputHex (uint8_t * pOutput, uint8_t length);
/*
   System Initialization
   Initializes Memory Manager, UART output, I2C connection, and HAL layer.
   Also initializes the ATCADevice object used by the CryptoAuthLib library.
 */
void systemInit()
{


    MM_Init();
    UART1_Init( 9600 );

    I2C1_Init( 100000 );
    secure_4_hal_init( 0x60, I2C1_Start, I2C1_Stop, I2C1_Wr, I2C1_Rd );
    Delay_1sec();

    atcab_init( &cfg_ateccx08a_i2c_default );
    memset (bufferIn, 0x00, 128);
    memset (bufferOut, 0x00, 128);

    LOG( "\r\n\r\nSystem Initialized\r\n" );
}


void main()
{
    systemInit();


    LOG( "\r\nStarting device testing...\r\n" );
 


    //First test - Reading serial number
    if (atcab_read_serial_number(bufferOut) == ATCA_SUCCESS)
    {
        LOG( "\r\n Serial number: " );
        outputHex (bufferOut, 9);
    }
    else  LOG( "\r\n Reading serial number failed..." );


    //Second test - Reading device config
    delay_ms (2500);
    memset (bufferOut, 0x00, 128);
    if (atcab_read_config_zone(ATECC508A, bufferOut) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n First 32 bytes of device configuration: " );
        outputHex (bufferOut, 32);
    }
    else  LOG( "\r\n Reading config zone failed..." );

    //Third test - Generating SHA digest for a given input message
    delay_ms (1500);
    memset (bufferOut, 0x00, 128);
    bufferIn [0] = 0x74;
    bufferIn [1] = 0xba;
    bufferIn [2] = 0x25;
    bufferIn [3] = 0x21;

    if (atcab_sha(4, bufferIn, bufferOut) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n SHA Digest of 0x74BA2521:  " );
        outputHex (bufferOut, 32);
        LOG("\r\n Expected value of digest: ");
        LOG("B1 6A A5 6B E3 88 0D 18 CD 41 E6 83 84 CF 1E C8 C1 76 80 C4 5A");
        LOG(" 02 B1 57 5D C1 51 89 23 AE 8B 0E");

    }
    else  LOG( "\r\n Generating SHA digest of the message failed..." );

    delay_ms (1500);
    


/*
  IMPORTANT: Uncommenting these lines permanently locks the config zone of the
  device.
  
  This is irreversible, so make sure configuration is set according to your
  needs. All the following tests won't work unless config zone is locked.

    additionalTests ();
    signatureTest ();
*/

    LOG( "\r\n\r\n--------------------------------------------" );
    LOG( "\r\nEND" );
}


/*
  IMPORTANT: Following functions permanently lock the config zone of the
  device. Do not use them if you are not sure what you're doing.

    
    This function WILL LOCK THE CONFIG ZONE. After that it will generate a
    random number, and output it. It will then generate a private key in slot 0,
    and output the corresponding public key.
*/
static void additionalTests ()
{
    //zone locking
    if (atcab_lock_config_zone(bufferOut) == ATCA_SUCCESS)
        LOG( "\r\n Config zone locked " );
    else  LOG( "\r\n Locking config zone failed or zone is already locked" );
/*
    Fourth test - Random number generator
    Will only return 0xFFFF0000FFFF0000 unless config zone is locked first.
*/
    memset (bufferOut, 0x00, 128);
    if (atcab_random(bufferOut) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n Generated random number: " );
        outputHex (bufferOut, 32);
    }
    else  LOG( "\r\n Generating random number failed..." );
/*
    Fifth test - Private key generation
    Will fail if the config zone is unlocked.
*/
    memset (bufferOut, 0x00, 128);
    if (atcab_genkey(0, bufferOut) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n Public key for slot 1: " );
        outputHex (bufferOut, 64);
    }
    else  LOG( "\r\n Private key generation failed..." );
}

/*
    This function creates a signature for a given message using existing public
    key, and then verifies if it is valid.
*/
static void signatureTest ()
{
    uint8_t messageBuffer  [ 32 ];
    uint8_t signatureBuffer  [ 64 ];
    uint8_t publicKeyBuffer  [ 64 ];
    uint8_t responseBuffer [ 3 ];
    
    memset (signatureBuffer, 0x00, 64);
    memset (publicKeyBuffer, 0x00, 64);
    memset (responseBuffer, 0x00, 3);
    //Message is 32 bytes of "0x42"
    memset (messageBuffer, 0x42, 32);


/*
    Reads the public key from slot 0. Public key must be generated first.
    This should be already done in additionalTests() function.
*/
    if (atcab_get_pubkey(0, publicKeyBuffer) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n Public key: " );
        outputHex (publicKeyBuffer, 64);
    }
    else  LOG( "\r\n Getting public key failed..." );

    //Generates the signature from input message, using key in slot 0.
    if (atcab_sign(0, messageBuffer, signatureBuffer) == ATCA_SUCCESS)
    {
        LOG( "\r\n\r\n Generating signature: " );
        outputHex (signatureBuffer, 64);
    }
    else  LOG( "\r\n Generating signature failed..." );

    //Verification of signature
    LOG( "\r\n\r\n Signature verification..." );
    if (atcab_verify_extern(messageBuffer, signatureBuffer, publicKeyBuffer, responseBuffer) == ATCA_SUCCESS)
    {
        if (responseBuffer [0])
             LOG( "\r\n Signature is valid " );
        else LOG( "\r\n Signature is invalid " );

    }
    else  LOG( "\r\n Verification operation failed..." );
}



//Help function for outputing data in hex format
void outputHex (uint8_t * pOutput, uint8_t length)
{
    uint8_t i;
    char text [10];
    for( i = 0; i < length; i++ )
    {
        ByteToHex( pOutput[ i ], text );
        LOG( Ltrim(text) );
        LOG( " " );
    }
}